import { HttpStatus, UnprocessableEntityException } from '@nestjs/common';
import { BaseDataService } from 'src/core/modules/data/service/base-data.service';

export class CheckDuplicateHelper {
  constructor(
    private dataService: BaseDataService<any>,
    private duplicateColumn: string[],
    private entity: any,
    private entityId?: string,
  ) {}

  async execute() {
    for (const column of this.duplicateColumn) {
      const queryBuilder = this.dataService
        .getRepository()
        .createQueryBuilder();
      queryBuilder.orWhere(
        `replace(trim(lower(${column})), '  ',' ') = :query`,
        {
          query: this.entity[column]
            ?.toLowerCase()
            .trim()
            .replace(/ +(?= )/g, ''),
        },
      );

      if (this.entityId) {
        queryBuilder.andWhere(`id Not In ('${this.entityId}')`);
      }

      const data_exists = await queryBuilder.getCount();
      if (data_exists > 0) {
        throw new UnprocessableEntityException({
          statusCode: HttpStatus.UNPROCESSABLE_ENTITY,
          message: `Entity with ${column} : ${this.entity[column]} already exist`,
          error: 'Unprocessable Entity',
        });
      }
    }
  }
}
